# encoding: utf-8
require 'fedux_org_stdlib/logic_converters/exceptions'

require 'fedux_org_stdlib/logic_converters/on_off_converter'
require 'fedux_org_stdlib/logic_converters/true_false_converter'
require 'fedux_org_stdlib/logic_converters/yes_no_converter'
require 'fedux_org_stdlib/logic_converters/y_n_converter'

module FeduxOrgStdlib
  module LogicConverters
    # Generic converter which passes arguments to other converters, e.g.
    # true/false, yes/no, on/off, y/n-converters
    class LogicConverter
      # @param [String, Symbol] type (true_false, yes_no, on_off)
      #   choose the type of converter which should be used
      def initialize(type)
        @converter = case type.to_s
                     when 'true_false'
                       TrueFalseConverter.new
                     when 'yes_no'
                       YesNoConverter.new
                     when 'on_off'
                       OnOffConverter.new
                     when 'y_n'
                       YNConverter.new
                     else
                       fail FeduxOrgStdlib::LogicConverters::Exceptions::InvalidType, "Unknown type \"#{type}\" for logic converter. Valid types are \"true_false\", \"yes_no\" or \"on_off\"."
                     end
      end

      # @param [true,false] value
      #   the logic value which should be converted
      # @return [String]
      #   the converted value
      def parse(value)
        @converter.parse(value)
      end
    end
  end
end
