# encoding: utf-8
require 'fedux_org_stdlib/logic_converters/exceptions'

module FeduxOrgStdlib
  module LogicConverters
    # Convert yes/no to true/false
    class YesNoConverter
      # @param [true,false] value
      #   the logic value which should be converted
      # @return [String]
      #   the converted value: true => yes, false => no, '' => nil
      #
      # @example Parse true
      #    converter = FeduxOrgStdlib::LogicConverters::OnOffConverter.new
      #    converter.parse( true ) # 'yes'
      #
      # @example Parse false
      #    converter = FeduxOrgStdlib::LogicConverters::OnOffConverter.new
      #    converter.parse( false ) # 'no'
      #
      def parse(value)
        case value
        when true
          'yes'
        when false
          'no'
        when ''
          nil
        else
          fail FeduxOrgStdlib::LogicConverters::Exceptions::InvalidValue, "Unknown logic value \"#{value}\"."
        end
      end
    end
  end
end
