# encoding: utf-8
module FeduxOrgStdlib
  module Models
    module Exceptions
      # internal error
      class InternalError < Exception; end

      # raised if one tries to delete an instance which
      # does not exist
      class InstanceNotFound < InternalError; end

      # raised if one tries to use a keyword whose use
      # is restricted
      class UnauthorizedUseOfKeyword < InternalError; end

      # raised if one uses invalid search criteria
      class InvalidSearchCriteria < InternalError; end

      # raised if the code which defines an import filter
      # is malformed
      class MethodNeedsToBeImplemented < InternalError; end

      # raised if one tries to use an unimplemented exception
      class ExceptionNeedsToBeImplemented < InternalError; end

      # raised if one forgot to store files at `path_to_instances`
      class NoImplementationsForModelFound < InternalError; end

      # raised if an file system error occured
      class FilesystemError < Exception; end

      # raise if a directory does not exist in filesystem
      class DirectoryDoesNotExist < FilesystemError; end

      # raise if a file does not exist in filesystem
      class FileDoesNotExist < FilesystemError; end
    end
  end
end
